//	test72.cc
//
//	An example user class for use by the "usertrackfilter" command.
//	This is just a silly example of how to do various things.
//
//	Build the shared object:
//		g4blmake
//	Then in your input.file:
//		usertrackfilter Name filter=test72 ...

#include <stdio.h>
#include <math.h>

#include "BLUserCode.hh"

/**	class test72 -- an example filter class for usertrackfilter
 **/
class test72 : public BLUserTrackFilter {
	int nKill;
	int nCreate;
public:
	test72() : BLUserTrackFilter() 
		{ nKill=0; nCreate=0; registerUserCode(); }
	~test72() { }
	virtual const char *getName() { return "test72"; }
	virtual void filter(G4Track *track, int eventID,
			std::vector<G4Track*> &secondaries, int verbose);
	virtual void setup(const char *init);
	virtual void complete(const char *init);
};
test72 test72Instance;	// registers itself

void test72::filter(G4Track *track, int eventID,
				std::vector<G4Track*> &secondaries, int verbose)
{
	// IMPORTANT: Don't modify Tune or Reference particles.
	if(eventID < 0) {
	    if(verbose > 0) printf("test72 Tune/Reference ignored.\n");
	    return;
	}

	// get particle PDGid (ignore anti-)
	int PDGid = getPDGid(track);
	if(PDGid < 0) PDGid = -PDGid;

	// kill all neutrinos
	if(PDGid == NU_E || PDGid == NU_MU || PDGid == NU_TAU) {
		if(verbose > 0) printf("test72 neutrino killed.\n");
		killTrack(track);
		++nKill;
		return;
	}

	// kill all input tracks with KE < 1 MeV
	double mass = getMass(PDGid);
	double KE = sqrt(track->GetMomentum().mag2() + mass*mass) - mass;
	if(KE < 1.0) {
		if(verbose > 0) printf("test72 KE<1.0 killed.\n");
		killTrack(track);
		++nKill;
		return;
	}

	// for charged particles, arbitrarily create three electrons, each
	// with 1% of the momentum of the input track
	if(PDGid == ELECTRON || PDGid == MUON || PDGid == PION || PDGid == KAON || 
								PDGid==PROTON) {
		if(verbose > 0) printf("test72 create 3 delta-rays.\n");
		G4ThreeVector momentum = track->GetMomentum() * 0.01;
		secondaries.push_back(constructTrack(track->GetPosition(),momentum,
					track->GetGlobalTime(),ELECTRON));
		secondaries.push_back(constructTrack(track->GetPosition(),momentum,
					track->GetGlobalTime(),ELECTRON));
		secondaries.push_back(constructTrack(track->GetPosition(),momentum,
					track->GetGlobalTime(),ELECTRON));
		nCreate += 3;
		// conserve overall momentum (yes, does NOT conserve energy)
		setMomentum(track,track->GetMomentum()-3.0*momentum);
	}

	// for gammas, halve the weight
	if(PDGid == GAMMA) {
		if(verbose > 0) printf("test72 gamma half-weight.\n");
		track->SetWeight(track->GetWeight()*0.5);
	}
}

void test72::setup(const char *init)
{
	// Initialize here. init can pass any desired string from the
	// "usertrackfilter" command line to this function.
	printf("test72::setup(%s)\n",init);
}

void test72::complete(const char *init)
{
	// print any summary here, and generally close up.
	printf("test72::complete  killed %d tracks, created %d tracks\n",
			nKill,nCreate);
}
